use std::path::PathBuf;

pub struct Config {
    pub database_url: String,
    pub jwt_secret: String,
    pub jwt_expiration: i64,
    pub master_key: String,
    pub storage_path: String,
    pub default_user_quota: i64,
}

impl Config {
    pub fn from_env() -> Self {
        let database_url = std::env::var("DATABASE_URL")
            .expect("DATABASE_URL must be set");

        let jwt_secret = std::env::var("JWT_SECRET")
            .expect("JWT_SECRET must be set");

        let jwt_expiration = std::env::var("JWT_EXPIRATION")
            .unwrap_or_else(|_| "86400".to_string()) // Default to 24 hours
            .parse::<i64>()
            .expect("JWT_EXPIRATION must be a valid number");

        let master_key = std::env::var("MASTER_KEY")
            .expect("MASTER_KEY must be set");

        let storage_path = std::env::var("STORAGE_PATH")
            .unwrap_or_else(|_| "./storage".to_string());

        let default_user_quota = std::env::var("DEFAULT_USER_QUOTA")
            .unwrap_or_else(|_| "5368709120".to_string()) // Default to 5GB
            .parse::<i64>()
            .expect("DEFAULT_USER_QUOTA must be a valid number");

        Self {
            database_url,
            jwt_secret,
            jwt_expiration,
            master_key,
            storage_path,
            default_user_quota,
        }
    }

    pub fn get_user_storage_path(&self, user_id: &str) -> PathBuf {
        let mut path = PathBuf::from(&self.storage_path);
        path.push(user_id);
        path
    }
}